package cs.fsu.iwander;

import java.io.File;
import java.util.Calendar;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.BatteryManager;
import android.os.Bundle;
import android.os.Environment;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.Toast;

public class IwanderLocationService extends Service implements LocationListener {

	public static Location homeLocation = null;
	public static final String LOCATION_UPDATE = "iwander.LOCATION_UPDATE";
	private static final String TAG = "cs.fsu.iwander";
	private LocationManager lm = null;
	private SharedPreferences sp = null;
	private BNContainer dataContainer = null;
	private float radius;
	private File root;

	public static int convertToGeoPoint(double degree){
		return (int)(degree * 1000000);
	}

	@Override
	public IBinder onBind(Intent arg0) {
		return null;
	}

	public RangeClassifier createRange(){
		RangeClassifier range = new RangeClassifier();

		range.setHighTemp(Integer.parseInt(sp.getString("hightemp", "1000")));
		range.setLowTemp(Integer.parseInt(sp.getString("lowtemp", "-1000")));

		return range;
	}


	public int onStartCommand(Intent intent, int flags, int startId){
		Log.d(TAG, "onStartCommand");
		registerLocationListener();

		homeLocation = new Location(LocationManager.GPS_PROVIDER);


		// Do we have a home location saved? if so retrieve it
		if(sp.getString("home_lat", "null").compareTo("null") != 0){
			homeLocation.setLatitude(Double.parseDouble(sp.getString("home_lat", "null")));
			homeLocation.setLongitude(Double.parseDouble(sp.getString("home_long", "null")));
			homeLocation.setAccuracy(Float.parseFloat(sp.getString("home_acc", "-1")));
		}
		else{
			homeLocation = lm.getLastKnownLocation(LocationManager.PASSIVE_PROVIDER);
		}


		if(homeLocation != null){
			Intent homeIntent = new Intent(LOCATION_UPDATE);
			homeIntent.putExtra("lat", homeLocation.getLatitude());
			homeIntent.putExtra("long", homeLocation.getLongitude());
			homeIntent.putExtra("acc", homeLocation.getAccuracy());
			homeIntent.putExtra("ishome", true);
			sendBroadcast(homeIntent);

			Log.v(TAG, "Started Service with homeLocation<" + homeLocation.getLatitude() + ">,<" + homeLocation.getLongitude() + "> accuracy " + homeLocation.getAccuracy());
		}
		else
			Log.w(TAG, "Location Unavalible");


		return 0;
	}

	public void onCreate(){
		super.onCreate();

		sp = PreferenceManager.getDefaultSharedPreferences(this);
		lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);

		root = Environment.getExternalStorageDirectory();

		new File(root, "iWander").mkdir();
	}

	private void registerLocationListener(){
		lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, Integer.parseInt(sp.getString("min_time", "0")), Integer.parseInt(sp.getString("min_distance", "0")), this);
		radius = Float.parseFloat(sp.getString("radius", "100"));
		dataContainer = new BNContainer(createRange());
		dataContainer.readFile(root.getAbsolutePath() + "/iWander/" + "12.data");
	}

	public void onDestroy(){
		lm.removeUpdates(this);
		sp = null;


		//dataContainer.writeFile(root.getAbsolutePath() + "/iWander/" + "12.data");

		Log.v(TAG, "Stopping Service");
		super.onDestroy();
	}


	@Override
	public void onLocationChanged(Location location) {

		// Only can about when outside allowable radius
		if(location != null && homeLocation.distanceTo(location) > radius){
			Intent weather = registerReceiver(null, new IntentFilter("alert.intent.action.ALERT"));

			ConditionNode condition = new ConditionNode();

			condition.setTime(DateUtils.parseMilitary());
			
			if(weather != null){
				condition.setTemp(weather.getIntExtra("temp",-9999));
				condition.setPrec(weather.getBooleanExtra("raning", false));
			}

			// Issue alert Here
			if(dataContainer.getProbability(condition) > Integer.parseInt(sp.getString("prob", "50")))
				issueAlert(condition);
			// Insert location into set
			else{

				// Gets Sitcky Intent for last Battery info Broadcast
				Intent battery = registerReceiver(null, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));

				Intent intent = new Intent(); 
				intent.putExtra("lat", location.getLatitude());
				intent.putExtra("long", location.getLongitude());
				intent.putExtra("acc", Math.round(location.getAccuracy()));
				intent.putExtra("ishome", false);
				intent.putExtra("prob", dataContainer.getProbability(condition));

				// Are we plugged into a power source
				if(battery.getIntExtra(BatteryManager.EXTRA_PLUGGED, 0) != 0){

					updateHomeLocation(location);	
					intent.putExtra("ishome", true);
				}

				intent.setAction(LOCATION_UPDATE);
				sendBroadcast(intent);
			}
			Log.i(TAG, "Lat<" + location.getLatitude() + "> Long<" + location.getLongitude() + ">");
		}

	}

	public void issueAlert(ConditionNode condtion){
		Intent intent = new Intent("alert.intent.action.ALERT");

		intent.putExtra("appname", "Alert From iWander");
		intent.putExtra("title", "Are you Lost?");
		intent.putExtra("button", "NO");
		intent.putExtra("sendsms", true);
		intent.putExtra("message", "Weather Stats here");

		getBaseContext().sendBroadcast(intent);
	}

	public void updateHomeLocation(Location location){
		Log.i(TAG, "Found new Home Location");
		Toast.makeText(this, "Home Location Set", Toast.LENGTH_SHORT).show();
		homeLocation = location;
		SharedPreferences.Editor editor = sp.edit();
		editor.putString("home_lat", location.getLatitude() + "");
		editor.putString("home_long", location.getLongitude() + "");
		editor.putString("home_acc", location.getAccuracy() + "");
		editor.commit();
	}

	@Override
	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub

	}

}
