package cs.fsu.iwander;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.CompoundButton;
import android.widget.ToggleButton;
import android.widget.CompoundButton.OnCheckedChangeListener;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapView;

import cs.fsu.R;

public class IwanderMapActivity extends MapActivity {
    private ToggleButton btnPower;
    private MapView map;
    
    private static final String TAG = "cs.fsu.IwanderMapActivity";
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        map = (MapView) findViewById(R.id.mapview);
        
        btnPower = (ToggleButton) findViewById(R.id.btnPower);
        
        btnPower.setOnCheckedChangeListener(new OnCheckedChangeListener(){

 			@Override
 			public void onCheckedChanged(CompoundButton buttonView,
 					boolean isChecked) {
 				
 				if(isChecked)
 					startService(new Intent(getBaseContext(),IwanderLocationService.class));
 				else{
 					stopService(new Intent(getBaseContext(),IwanderLocationService.class));
 				}
 				
 			}
         	
         });
    }
    
    public void onResume(){
    	super.onResume();
    	
    	registerReceiver(locationUpdater, new IntentFilter(IwanderLocationService.LOCATION_UPDATE));
    }
    
    private void centerMap(GeoPoint point){
    	map.getController().setCenter(point);
    	map.getController().setZoom(17);
    }
    
    
    public void onPause(){
    	super.onPause();
    	
    	unregisterReceiver(locationUpdater);
    }
    
    
	/* Creates the menu items */
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(Menu.NONE, Menu.FIRST, Menu.NONE, "Monitor Settings").setIcon(android.R.drawable.ic_menu_preferences);
		return true;
	}

	/* Handles item selections */
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent = new Intent();
		switch (item.getItemId()) {

		case Menu.FIRST:
			intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			intent.setClass(getBaseContext(), PrefSettingsActivity.class);
			startActivity(intent);
			return true;
		}
		return false;
	}

	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}
	
	BroadcastReceiver locationUpdater = new BroadcastReceiver(){
		
		@Override
		public void onReceive(Context context, Intent intent) {
			Log.i(TAG, "Received Intent to draw home " + intent.getDoubleExtra("lat", 0) + intent.getDoubleExtra("long",0));
			
			
			SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
			
			int lat = (int)(intent.getDoubleExtra("lat", 0) * 1000000),
				lon = (int)(intent.getDoubleExtra("long", 0) * 1000000),
				acc = (int)(intent.getIntExtra("acc", -1)),
				radius = Integer.parseInt(prefs.getString("radius", "0"));
			
			
			boolean ishome = intent.getBooleanExtra("ishome", false);
			
			GeoPoint home = new GeoPoint(lat,lon); 
			
			Log.i(TAG, "Drawing Home point " + home.getLatitudeE6() + " " + home.getLongitudeE6() + " radius (meters) = " + radius);
			
			centerMap(home);
			
			if(ishome)
				map.getOverlays().add(new HomeOverlay(getResources().getDrawable(R.drawable.home), home, radius, ishome));
			else
				map.getOverlays().add(new HomeOverlay(getResources().getDrawable(R.drawable.person), home, acc, ishome));
			
			
		
		}
		
	};
    
    
}