package cs.fsu.iwander;

import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.util.Log;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;
import com.google.android.maps.Projection;

public class LocationOverlay extends ItemizedOverlay<OverlayItem> {

	private Drawable marker = null;
	private OverlayItem item = null;
	private int lat, lon, acc, rad;
	private GeoPoint loc;
	
	private static final String TAG = "cs.fsu.iWander.LocationItemizedOverlay";
	
	
	public LocationOverlay(Drawable marker, GeoPoint location, int accuracy, int radius) {
		super(marker);
		this.marker = marker;

	
		loc = location;
		lat = location.getLatitudeE6();
		lon = location.getLongitudeE6();
		acc = accuracy;
		rad = radius;
		
		
		item = new OverlayItem(location, "Home", "Stay Put");

		populate();
	}

	@Override
	protected OverlayItem createItem(int i) {
		return (item);
	}

	@Override
	public int size() {
		return 1;
	}

	public void draw(Canvas canvas, MapView mapView,
			boolean shadow) {
		super.draw(canvas, mapView, shadow);
		
		
		
		boundCenterBottom(marker);
	
		
		Projection projection = mapView.getProjection();
		Point home = new Point();
		
		
		int pixels = metersToRadius(rad, mapView, lat);
		
		projection.toPixels(loc, home);
		
		Paint paint = new Paint();
		paint.setAntiAlias(true);
		paint.setStrokeWidth(2.0f);
		
		paint.setColor(0xff6666ff);
		paint.setAlpha(100);
		
		Log.i(TAG, "pixels = " + pixels);
		
		
		canvas.drawCircle(home.x, home.y, pixels, paint);
	}
	
	public static int metersToRadius(float meters, MapView map, double latitude) {
	    return Math.abs((int) (map.getProjection().metersToEquatorPixels(meters) * (1/ Math.cos(Math.toRadians(latitude)))));         
	}


	//private ArrayList<OverlayItem> mOverlays = new ArrayList<OverlayItem>();
	//private Context context;



	/*
	public LocationItemizedOverlay(Drawable defaultMarker) {
		super(boundCenterBottom(defaultMarker));
	}

	public LocationItemizedOverlay(Drawable defaultMarker, Context context) {
		  super(defaultMarker);
		  this.context = context;
		}


	@Override
	protected OverlayItem createItem(int i) {
		return mOverlays.get(i);
	}

	@Override
	public int size() {
		// TODO Auto-generated method stub
		return mOverlays.size();
	}

	public void addOverlay(OverlayItem overlay) {
	    mOverlays.add(overlay);
	    populate();
	}

	protected boolean onTap(int index) {
		  OverlayItem item = mOverlays.get(index);
		  AlertDialog.Builder dialog = new AlertDialog.Builder(context);
		  dialog.setTitle(item.getTitle());
		  dialog.setMessage(item.getSnippet());
		  dialog.show();
		  return true;
		}

	 */
}
